package sun.security.x509;

import java.security.MessageDigest;
import sun.security.provider.X509Factory;
import java.security.cert.Certificate;
import sun.security.util.DerInputStream;
import java.util.ArrayList;
import java.util.Collections;
import sun.security.util.ObjectIdentifier;
import java.util.Iterator;
import java.util.TreeSet;
import javax.security.auth.x500.X500Principal;
import java.security.Principal;
import java.math.BigInteger;
import sun.misc.HexDumpEncoder;
import java.util.Enumeration;
import java.security.cert.CertificateParsingException;
import java.security.cert.CertificateNotYetValidException;
import java.security.cert.CertificateExpiredException;
import java.util.Date;
import sun.security.util.DerOutputStream;
import java.security.spec.AlgorithmParameterSpec;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.InvalidAlgorithmParameterException;
import java.security.ProviderException;
import sun.security.util.SignatureUtil;
import java.security.Signature;
import java.security.SignatureException;
import java.security.NoSuchProviderException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateEncodingException;
import java.io.OutputStream;
import sun.security.util.Pem;
import java.io.ByteArrayOutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.security.cert.CertificateException;
import sun.security.util.DerValue;
import java.util.concurrent.ConcurrentHashMap;
import java.security.PublicKey;
import java.util.Set;
import java.util.List;
import java.util.Collection;
import sun.security.util.DerEncoder;
import java.security.cert.X509Certificate;

public class X509CertImpl extends X509Certificate implements DerEncoder
{
    private static final long serialVersionUID = -3457612960190864406L;
    private static final String DOT = ".";
    public static final String NAME = "x509";
    public static final String INFO = "info";
    public static final String ALG_ID = "algorithm";
    public static final String SIGNATURE = "signature";
    public static final String SIGNED_CERT = "signed_cert";
    public static final String SUBJECT_DN = "x509.info.subject.dname";
    public static final String ISSUER_DN = "x509.info.issuer.dname";
    public static final String SERIAL_ID = "x509.info.serialNumber.number";
    public static final String PUBLIC_KEY = "x509.info.key.value";
    public static final String VERSION = "x509.info.version.number";
    public static final String SIG_ALG = "x509.algorithm";
    public static final String SIG = "x509.signature";
    private boolean readOnly;
    private byte[] signedCert;
    protected X509CertInfo info;
    protected AlgorithmId algId;
    protected byte[] signature;
    private static final String KEY_USAGE_OID = "2.5.29.15";
    private static final String EXTENDED_KEY_USAGE_OID = "2.5.29.37";
    private static final String BASIC_CONSTRAINT_OID = "2.5.29.19";
    private static final String SUBJECT_ALT_NAME_OID = "2.5.29.17";
    private static final String ISSUER_ALT_NAME_OID = "2.5.29.18";
    private static final String AUTH_INFO_ACCESS_OID = "1.3.6.1.5.5.7.1.1";
    private static final int NUM_STANDARD_KEY_USAGE = 9;
    private Collection<List<?>> subjectAlternativeNames;
    private Collection<List<?>> issuerAlternativeNames;
    private List<String> extKeyUsage;
    private Set<AccessDescription> authInfoAccess;
    private PublicKey verifiedPublicKey;
    private String verifiedProvider;
    private boolean verificationResult;
    private ConcurrentHashMap<String, String> fingerprints;
    
    public X509CertImpl() {
        this.readOnly = false;
        this.signedCert = null;
        this.info = null;
        this.algId = null;
        this.signature = null;
        this.fingerprints = new ConcurrentHashMap(2);
    }
    
    public X509CertImpl(byte[] encoding) throws CertificateException {
        this.readOnly = false;
        this.signedCert = null;
        this.info = null;
        this.algId = null;
        this.signature = null;
        this.fingerprints = new ConcurrentHashMap(2);
        try {
            this.parse(new DerValue(encoding));
        }
        catch (IOException ex) {
            this.signedCert = null;
            throw new CertificateException("Unable to initialize, " + ex, ex);
        }
    }
    
    public X509CertImpl(InputStream in) throws CertificateException {
        this.readOnly = false;
        this.signedCert = null;
        this.info = null;
        this.algId = null;
        this.signature = null;
        this.fingerprints = new ConcurrentHashMap(2);
        BufferedInputStream in2 = new BufferedInputStream(in);
        DerValue rfc1421Cert;
        try {
            in2.mark(Integer.MAX_VALUE);
            rfc1421Cert = this.readRFC1421Cert(in2);
        }
        catch (IOException ex) {
            try {
                in2.reset();
                rfc1421Cert = new DerValue(in2);
            }
            catch (IOException cause) {
                throw new CertificateException("Input stream must be either DER-encoded bytes or RFC1421 hex-encoded DER-encoded bytes: " + cause.getMessage(), cause);
            }
        }
        try {
            this.parse(rfc1421Cert);
        }
        catch (IOException ex2) {
            this.signedCert = null;
            throw new CertificateException("Unable to parse DER value of certificate, " + ex2, ex2);
        }
    }
    
    private DerValue readRFC1421Cert(InputStream in) throws IOException {
        DerValue derValue = null;
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(in, "ASCII"));
        String line;
        try {
            line = bufferedReader.readLine();
        }
        catch (IOException ex) {
            throw new IOException("Unable to read InputStream: " + ex.getMessage());
        }
        if (line.equals("-----BEGIN CERTIFICATE-----")) {
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            try {
                String line2;
                while ((line2 = bufferedReader.readLine()) != null) {
                    if (line2.equals("-----END CERTIFICATE-----")) {
                        derValue = new DerValue(byteArrayOutputStream.toByteArray());
                        break;
                    }
                    byteArrayOutputStream.write(Pem.decode(line2));
                }
            }
            catch (IOException ex2) {
                throw new IOException("Unable to read InputStream: " + ex2.getMessage());
            }
            return derValue;
        }
        throw new IOException("InputStream is not RFC1421 hex-encoded DER bytes");
    }
    
    public X509CertImpl(X509CertInfo info) {
        this.readOnly = false;
        this.signedCert = null;
        this.info = null;
        this.algId = null;
        this.signature = null;
        this.fingerprints = new ConcurrentHashMap(2);
        this.info = info;
    }
    
    public X509CertImpl(DerValue derValue) throws CertificateException {
        this.readOnly = false;
        this.signedCert = null;
        this.info = null;
        this.algId = null;
        this.signature = null;
        this.fingerprints = new ConcurrentHashMap(2);
        try {
            this.parse(derValue);
        }
        catch (IOException ex) {
            this.signedCert = null;
            throw new CertificateException("Unable to initialize, " + ex, ex);
        }
    }
    
    public void encode(OutputStream outputStream) throws CertificateEncodingException {
        if (this.signedCert == null) {
            throw new CertificateEncodingException("Null certificate to encode");
        }
        try {
            outputStream.write((byte[])this.signedCert.clone());
        }
        catch (IOException ex) {
            throw new CertificateEncodingException(ex.toString());
        }
        return;
    }
    
    public void derEncode(OutputStream outputStream) throws IOException {
        if (this.signedCert == null) {
            throw new IOException("Null certificate to encode");
        }
        outputStream.write((byte[])this.signedCert.clone());
        return;
    }
    
    @Override
    public byte[] getEncoded() throws CertificateEncodingException {
        return (byte[])this.getEncodedInternal().clone();
    }
    
    public byte[] getEncodedInternal() throws CertificateEncodingException {
        if (this.signedCert == null) {
            throw new CertificateEncodingException("Null certificate to encode");
        }
        return this.signedCert;
    }
    
    @Override
    public void verify(PublicKey publicKey) throws CertificateException, NoSuchAlgorithmException, InvalidKeyException, NoSuchProviderException, SignatureException {
        this.verify(publicKey, "");
    }
    
    @Override
    public synchronized void verify(PublicKey verifiedPublicKey, String s) throws CertificateException, NoSuchAlgorithmException, InvalidKeyException, NoSuchProviderException, SignatureException {
        if (s == null) {
            s = "";
        }
        if (this.verifiedPublicKey != null && this.verifiedPublicKey.equals(verifiedPublicKey) && s.equals(this.verifiedProvider)) {
            if (this.verificationResult) {
                return;
            }
            throw new SignatureException("Signature does not match.");
        }
        else {
            if (this.signedCert == null) {
                throw new CertificateEncodingException("Uninitialized certificate");
            }
            String name = this.algId.getName();
            Signature s2;
            if (s.length() == 0) {
                s2 = Signature.getInstance(name);
            }
            else {
                s2 = Signature.getInstance(name, s);
            }
            try {
                SignatureUtil.initVerifyWithParam(s2, verifiedPublicKey, SignatureUtil.getParamSpec(name, this.getSigAlgParams()));
            }
            catch (ProviderException ex) {
                throw new CertificateException(ex.getMessage(), ex.getCause());
            }
            catch (InvalidAlgorithmParameterException cause) {
                throw new CertificateException(cause);
            }
            byte[] encodedInfo = this.info.getEncodedInfo();
            s2.update(encodedInfo, 0, encodedInfo.length);
            this.verificationResult = s2.verify(this.signature);
            this.verifiedPublicKey = verifiedPublicKey;
            this.verifiedProvider = s;
            if (!this.verificationResult) {
                throw new SignatureException("Signature does not match.");
            }
            return;
        }
    }
    
    @Override
    public synchronized void verify(PublicKey publicKey, Provider provider) throws CertificateException, NoSuchAlgorithmException, InvalidKeyException, SignatureException {
        if (this.signedCert == null) {
            throw new CertificateEncodingException("Uninitialized certificate");
        }
        String name = this.algId.getName();
        Signature s;
        if (provider == null) {
            s = Signature.getInstance(name);
        }
        else {
            s = Signature.getInstance(name, provider);
        }
        try {
            SignatureUtil.initVerifyWithParam(s, publicKey, SignatureUtil.getParamSpec(name, this.getSigAlgParams()));
        }
        catch (ProviderException ex) {
            throw new CertificateException(ex.getMessage(), ex.getCause());
        }
        catch (InvalidAlgorithmParameterException cause) {
            throw new CertificateException(cause);
        }
        byte[] encodedInfo = this.info.getEncodedInfo();
        s.update(encodedInfo, 0, encodedInfo.length);
        this.verificationResult = s.verify(this.signature);
        this.verifiedPublicKey = publicKey;
        if (!this.verificationResult) {
            throw new SignatureException("Signature does not match.");
        }
        return;
    }
    
    public void sign(PrivateKey privateKey, String s) throws CertificateException, NoSuchAlgorithmException, InvalidKeyException, NoSuchProviderException, SignatureException {
        this.sign(privateKey, s, null);
    }
    
    public void sign(PrivateKey privateKey, String s, String s2) throws CertificateException, NoSuchAlgorithmException, InvalidKeyException, NoSuchProviderException, SignatureException {
        try {
            this.sign(privateKey, null, s, s2);
        }
        catch (InvalidAlgorithmParameterException cause) {
            throw new SignatureException(cause);
        }
    }
    
    public void sign(PrivateKey key, AlgorithmParameterSpec params, String s, String provider) throws CertificateException, NoSuchAlgorithmException, InvalidKeyException, InvalidAlgorithmParameterException, NoSuchProviderException, SignatureException {
        try {
            if (this.readOnly) {
                throw new CertificateEncodingException("cannot over-write existing certificate");
            }
            Signature s2;
            if (provider == null || provider.length() == 0) {
                s2 = Signature.getInstance(s);
            }
            else {
                s2 = Signature.getInstance(s, provider);
            }
            SignatureUtil.initSignWithParam(s2, key, params, null);
            if (params != null) {
                this.algId = AlgorithmId.get(s2.getParameters());
            }
            else {
                this.algId = AlgorithmId.get(s);
            }
            DerOutputStream o = new DerOutputStream();
            DerOutputStream o2 = new DerOutputStream();
            this.info.encode(o2);
            byte[] byteArray = o2.toByteArray();
            this.algId.encode(o2);
            s2.update(byteArray, 0, byteArray.length);
            o2.putBitString(this.signature = s2.sign());
            o.write((byte)48, o2);
            this.signedCert = o.toByteArray();
            this.readOnly = true;
        }
        catch (IOException ex) {
            throw new CertificateEncodingException(ex.toString());
        }
    }
    
    @Override
    public void checkValidity() throws CertificateExpiredException, CertificateNotYetValidException {
        this.checkValidity(new Date());
    }
    
    @Override
    public void checkValidity(Date date) throws CertificateExpiredException, CertificateNotYetValidException {
        CertificateValidity certificateValidity;
        try {
            certificateValidity = (CertificateValidity)this.info.get("validity");
        }
        catch (Exception ex) {
            throw new CertificateNotYetValidException("Incorrect validity period");
        }
        if (certificateValidity == null) {
            throw new CertificateNotYetValidException("Null validity period");
        }
        certificateValidity.valid(date);
        return;
    }
    
    public Object get(String s) throws CertificateParsingException {
        X509AttributeName x509AttributeName = new X509AttributeName(s);
        String prefix = x509AttributeName.getPrefix();
        if (!prefix.equalsIgnoreCase("x509")) {
            throw new CertificateParsingException("Invalid root of attribute name, expected [x509], received [" + prefix + "]");
        }
        X509AttributeName x509AttributeName2 = new X509AttributeName(x509AttributeName.getSuffix());
        String prefix2 = x509AttributeName2.getPrefix();
        if (prefix2.equalsIgnoreCase("info")) {
            if (this.info == null) {
                return null;
            }
            if (x509AttributeName2.getSuffix() != null) {
                try {
                    return this.info.get(x509AttributeName2.getSuffix());
                }
                catch (IOException ex) {
                    throw new CertificateParsingException(ex.toString());
                }
                catch (CertificateException ex2) {
                    throw new CertificateParsingException(ex2.toString());
                }
            }
            else {
                return this.info;
            }
        }
        else {
            if (prefix2.equalsIgnoreCase("algorithm")) {
                return this.algId;
            }
            if (prefix2.equalsIgnoreCase("signature")) {
                if (this.signature != null) {
                    return this.signature.clone();
                }
                return null;
            }
            else if (prefix2.equalsIgnoreCase("signed_cert")) {
                if (this.signedCert != null) {
                    return this.signedCert.clone();
                }
                return null;
            }
            else {
                throw new CertificateParsingException("Attribute name not recognized or get() not allowed for the same: " + prefix2);
            }
        }
    }
    
    public void set(String s, Object o) throws CertificateException, IOException {
        if (this.readOnly) {
            throw new CertificateException("cannot over-write existing certificate");
        }
        X509AttributeName x509AttributeName = new X509AttributeName(s);
        String prefix = x509AttributeName.getPrefix();
        if (!prefix.equalsIgnoreCase("x509")) {
            throw new CertificateException("Invalid root of attribute name, expected [x509], received " + prefix);
        }
        X509AttributeName x509AttributeName2 = new X509AttributeName(x509AttributeName.getSuffix());
        String prefix2 = x509AttributeName2.getPrefix();
        if (prefix2.equalsIgnoreCase("info")) {
            if (x509AttributeName2.getSuffix() == null) {
                if (!(o instanceof X509CertInfo)) {
                    throw new CertificateException("Attribute value should be of type X509CertInfo.");
                }
                this.info = (X509CertInfo)o;
                this.signedCert = null;
            }
            else {
                this.info.set(x509AttributeName2.getSuffix(), o);
                this.signedCert = null;
            }
            return;
        }
        else {
            throw new CertificateException("Attribute name not recognized or set() not allowed for the same: " + prefix2);
        }
    }
    
    public void delete(String s) throws CertificateException, IOException {
        if (this.readOnly) {
            throw new CertificateException("cannot over-write existing certificate");
        }
        X509AttributeName x509AttributeName = new X509AttributeName(s);
        String prefix = x509AttributeName.getPrefix();
        if (!prefix.equalsIgnoreCase("x509")) {
            throw new CertificateException("Invalid root of attribute name, expected [x509], received " + prefix);
        }
        X509AttributeName x509AttributeName2 = new X509AttributeName(x509AttributeName.getSuffix());
        String prefix2 = x509AttributeName2.getPrefix();
        if (prefix2.equalsIgnoreCase("info")) {
            if (x509AttributeName2.getSuffix() != null) {
                this.info = null;
            }
            else {
                this.info.delete(x509AttributeName2.getSuffix());
            }
        }
        else if (prefix2.equalsIgnoreCase("algorithm")) {
            this.algId = null;
        }
        else if (prefix2.equalsIgnoreCase("signature")) {
            this.signature = null;
        }
        else if (prefix2.equalsIgnoreCase("signed_cert")) {
            this.signedCert = null;
        }
        else {
            throw new CertificateException("Attribute name not recognized or delete() not allowed for the same: " + prefix2);
        }
        return;
    }
    
    public Enumeration<String> getElements() {
        AttributeNameEnumeration attributeNameEnumeration = new AttributeNameEnumeration();
        attributeNameEnumeration.addElement("x509.info");
        attributeNameEnumeration.addElement("x509.algorithm");
        attributeNameEnumeration.addElement("x509.signature");
        attributeNameEnumeration.addElement("x509.signed_cert");
        return attributeNameEnumeration.elements();
    }
    
    public String getName() {
        return "x509";
    }
    
    @Override
    public String toString() {
        if (this.info == null || this.algId == null || this.signature == null) {
            return "";
        }
        StringBuilder sb = new StringBuilder();
        sb.append("[\n");
        sb.append(this.info.toString() + "\n");
        sb.append("  Algorithm: [" + this.algId.toString() + "]\n");
        sb.append("  Signature:\n" + new HexDumpEncoder().encodeBuffer(this.signature));
        sb.append("\n]");
        return sb.toString();
    }
    
    @Override
    public PublicKey getPublicKey() {
        if (this.info == null) {
            return null;
        }
        try {
            return (PublicKey)this.info.get("key.value");
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    @Override
    public int getVersion() {
        if (this.info == null) {
            return -1;
        }
        try {
            return (int)this.info.get("version.number") + 1;
        }
        catch (Exception ex) {
            return -1;
        }
    }
    
    @Override
    public BigInteger getSerialNumber() {
        SerialNumber serialNumberObject = this.getSerialNumberObject();
        return (serialNumberObject != null) ? serialNumberObject.getNumber() : null;
    }
    
    public SerialNumber getSerialNumberObject() {
        if (this.info == null) {
            return null;
        }
        try {
            return (SerialNumber)this.info.get("serialNumber.number");
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    @Override
    public Principal getSubjectDN() {
        if (this.info == null) {
            return null;
        }
        try {
            return (Principal)this.info.get("subject.dname");
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    @Override
    public X500Principal getSubjectX500Principal() {
        if (this.info == null) {
            return null;
        }
        try {
            return (X500Principal)this.info.get("subject.x500principal");
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    @Override
    public Principal getIssuerDN() {
        if (this.info == null) {
            return null;
        }
        try {
            return (Principal)this.info.get("issuer.dname");
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    @Override
    public X500Principal getIssuerX500Principal() {
        if (this.info == null) {
            return null;
        }
        try {
            return (X500Principal)this.info.get("issuer.x500principal");
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    @Override
    public Date getNotBefore() {
        if (this.info == null) {
            return null;
        }
        try {
            return (Date)this.info.get("validity.notBefore");
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    @Override
    public Date getNotAfter() {
        if (this.info == null) {
            return null;
        }
        try {
            return (Date)this.info.get("validity.notAfter");
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    @Override
    public byte[] getTBSCertificate() throws CertificateEncodingException {
        if (this.info != null) {
            return this.info.getEncodedInfo();
        }
        throw new CertificateEncodingException("Uninitialized certificate");
    }
    
    @Override
    public byte[] getSignature() {
        if (this.signature == null) {
            return null;
        }
        return (byte[])this.signature.clone();
    }
    
    @Override
    public String getSigAlgName() {
        if (this.algId == null) {
            return null;
        }
        return this.algId.getName();
    }
    
    @Override
    public String getSigAlgOID() {
        if (this.algId == null) {
            return null;
        }
        return this.algId.getOID().toString();
    }
    
    @Override
    public byte[] getSigAlgParams() {
        if (this.algId == null) {
            return null;
        }
        try {
            return this.algId.getEncodedParams();
        }
        catch (IOException ex) {
            return null;
        }
    }
    
    @Override
    public boolean[] getIssuerUniqueID() {
        if (this.info == null) {
            return null;
        }
        try {
            UniqueIdentity uniqueIdentity = (UniqueIdentity)this.info.get("issuerID");
            if (uniqueIdentity == null) {
                return null;
            }
            return uniqueIdentity.getId();
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    @Override
    public boolean[] getSubjectUniqueID() {
        if (this.info == null) {
            return null;
        }
        try {
            UniqueIdentity uniqueIdentity = (UniqueIdentity)this.info.get("subjectID");
            if (uniqueIdentity == null) {
                return null;
            }
            return uniqueIdentity.getId();
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    public KeyIdentifier getAuthKeyId() {
        AuthorityKeyIdentifierExtension authorityKeyIdentifierExtension = this.getAuthorityKeyIdentifierExtension();
        if (authorityKeyIdentifierExtension != null) {
            try {
                return (KeyIdentifier)authorityKeyIdentifierExtension.get("key_id");
            }
            catch (IOException ex) {}
        }
        return null;
    }
    
    public KeyIdentifier getSubjectKeyId() {
        SubjectKeyIdentifierExtension subjectKeyIdentifierExtension = this.getSubjectKeyIdentifierExtension();
        if (subjectKeyIdentifierExtension != null) {
            try {
                return subjectKeyIdentifierExtension.get("key_id");
            }
            catch (IOException ex) {}
        }
        return null;
    }
    
    public AuthorityKeyIdentifierExtension getAuthorityKeyIdentifierExtension() {
        return (AuthorityKeyIdentifierExtension)this.getExtension(PKIXExtensions.AuthorityKey_Id);
    }
    
    public BasicConstraintsExtension getBasicConstraintsExtension() {
        return (BasicConstraintsExtension)this.getExtension(PKIXExtensions.BasicConstraints_Id);
    }
    
    public CertificatePoliciesExtension getCertificatePoliciesExtension() {
        return (CertificatePoliciesExtension)this.getExtension(PKIXExtensions.CertificatePolicies_Id);
    }
    
    public ExtendedKeyUsageExtension getExtendedKeyUsageExtension() {
        return (ExtendedKeyUsageExtension)this.getExtension(PKIXExtensions.ExtendedKeyUsage_Id);
    }
    
    public IssuerAlternativeNameExtension getIssuerAlternativeNameExtension() {
        return (IssuerAlternativeNameExtension)this.getExtension(PKIXExtensions.IssuerAlternativeName_Id);
    }
    
    public NameConstraintsExtension getNameConstraintsExtension() {
        return (NameConstraintsExtension)this.getExtension(PKIXExtensions.NameConstraints_Id);
    }
    
    public PolicyConstraintsExtension getPolicyConstraintsExtension() {
        return (PolicyConstraintsExtension)this.getExtension(PKIXExtensions.PolicyConstraints_Id);
    }
    
    public PolicyMappingsExtension getPolicyMappingsExtension() {
        return (PolicyMappingsExtension)this.getExtension(PKIXExtensions.PolicyMappings_Id);
    }
    
    public PrivateKeyUsageExtension getPrivateKeyUsageExtension() {
        return (PrivateKeyUsageExtension)this.getExtension(PKIXExtensions.PrivateKeyUsage_Id);
    }
    
    public SubjectAlternativeNameExtension getSubjectAlternativeNameExtension() {
        return (SubjectAlternativeNameExtension)this.getExtension(PKIXExtensions.SubjectAlternativeName_Id);
    }
    
    public SubjectKeyIdentifierExtension getSubjectKeyIdentifierExtension() {
        return (SubjectKeyIdentifierExtension)this.getExtension(PKIXExtensions.SubjectKey_Id);
    }
    
    public CRLDistributionPointsExtension getCRLDistributionPointsExtension() {
        return (CRLDistributionPointsExtension)this.getExtension(PKIXExtensions.CRLDistributionPoints_Id);
    }
    
    @Override
    public boolean hasUnsupportedCriticalExtension() {
        if (this.info == null) {
            return false;
        }
        try {
            CertificateExtensions certificateExtensions = (CertificateExtensions)this.info.get("extensions");
            return certificateExtensions != null && certificateExtensions.hasUnsupportedCriticalExtension();
        }
        catch (Exception ex) {
            return false;
        }
    }
    
    @Override
    public Set<String> getCriticalExtensionOIDs() {
        if (this.info == null) {
            return null;
        }
        try {
            CertificateExtensions certificateExtensions = (CertificateExtensions)this.info.get("extensions");
            if (certificateExtensions == null) {
                return null;
            }
            TreeSet<String> set = new TreeSet();
            for (Extension extension : certificateExtensions.getAllExtensions()) {
                if (extension.isCritical()) {
                    set.add(extension.getExtensionId().toString());
                }
                else {
                    continue;
                }
            }
            return set;
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    @Override
    public Set<String> getNonCriticalExtensionOIDs() {
        if (this.info == null) {
            return null;
        }
        try {
            CertificateExtensions certificateExtensions = (CertificateExtensions)this.info.get("extensions");
            if (certificateExtensions == null) {
                return null;
            }
            TreeSet<String> set = new TreeSet();
            for (Extension extension : certificateExtensions.getAllExtensions()) {
                if (!extension.isCritical()) {
                    set.add(extension.getExtensionId().toString());
                }
                else {
                    continue;
                }
            }
            set.addAll(certificateExtensions.getUnparseableExtensions().keySet());
            return set;
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    public Extension getExtension(ObjectIdentifier obj) {
        if (this.info == null) {
            return null;
        }
        try {
            CertificateExtensions certificateExtensions;
            try {
                certificateExtensions = (CertificateExtensions)this.info.get("extensions");
            }
            catch (CertificateException ex) {
                return null;
            }
            if (certificateExtensions == null) {
                return null;
            }
            Extension extension = certificateExtensions.getExtension(obj.toString());
            if (extension != null) {
                return extension;
            }
            for (Extension extension2 : certificateExtensions.getAllExtensions()) {
                if (extension2.getExtensionId().equals(obj)) {
                    return extension2;
                }
                continue;
            }
            return null;
        }
        catch (IOException ex2) {
            return null;
        }
    }
    
    public Extension getUnparseableExtension(ObjectIdentifier objectIdentifier) {
        if (this.info == null) {
            return null;
        }
        try {
            CertificateExtensions certificateExtensions;
            try {
                certificateExtensions = (CertificateExtensions)this.info.get("extensions");
            }
            catch (CertificateException ex) {
                return null;
            }
            if (certificateExtensions == null) {
                return null;
            }
            return (Extension)certificateExtensions.getUnparseableExtensions().get(objectIdentifier.toString());
        }
        catch (IOException ex2) {
            return null;
        }
    }
    
    @Override
    public byte[] getExtensionValue(String oid) {
        try {
            ObjectIdentifier obj = new ObjectIdentifier(oid);
            String name = OIDMap.getName(obj);
            Extension extension = null;
            CertificateExtensions certificateExtensions = (CertificateExtensions)this.info.get("extensions");
            if (name == null) {
                if (certificateExtensions == null) {
                    return null;
                }
                for (Extension extension2 : certificateExtensions.getAllExtensions()) {
                    if (extension2.getExtensionId().equals(obj)) {
                        extension = extension2;
                        break;
                    }
                    continue;
                }
            }
            else {
                try {
                    extension = (Extension)this.get(name);
                }
                catch (CertificateException ex) {}
            }
            if (extension == null) {
                if (certificateExtensions != null) {
                    extension = (Extension)certificateExtensions.getUnparseableExtensions().get(oid);
                }
                if (extension == null) {
                    return null;
                }
            }
            byte[] extensionValue = extension.getExtensionValue();
            if (extensionValue == null) {
                return null;
            }
            DerOutputStream o = new DerOutputStream();
            o.putOctetString(extensionValue);
            return o.toByteArray();
        }
        catch (Exception ex2) {
            return null;
        }
    }
    
    @Override
    public boolean[] getKeyUsage() {
        try {
            String name = OIDMap.getName(PKIXExtensions.KeyUsage_Id);
            if (name == null) {
                return null;
            }
            KeyUsageExtension keyUsageExtension = (KeyUsageExtension)this.get(name);
            if (keyUsageExtension == null) {
                return null;
            }
            boolean[] bits = keyUsageExtension.getBits();
            if (bits.length < 9) {
                boolean[] array = new boolean[9];
                System.arraycopy(bits, 0, array, 0, bits.length);
                bits = array;
                return bits;
            }
            return bits;
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    @Override
    public synchronized List<String> getExtendedKeyUsage() throws CertificateParsingException {
        if (this.readOnly && this.extKeyUsage != null) {
            return this.extKeyUsage;
        }
        ExtendedKeyUsageExtension extendedKeyUsageExtension = this.getExtendedKeyUsageExtension();
        if (extendedKeyUsageExtension == null) {
            return null;
        }
        return this.extKeyUsage = Collections.unmodifiableList(extendedKeyUsageExtension.getExtendedKeyUsage());
    }
    
    public static List<String> getExtendedKeyUsage(X509Certificate x509Certificate) throws CertificateParsingException {
        try {
            byte[] extensionValue = x509Certificate.getExtensionValue("2.5.29.37");
            if (extensionValue == null) {
                return null;
            }
            return Collections.unmodifiableList(new ExtendedKeyUsageExtension(Boolean.FALSE, new DerValue(extensionValue).getOctetString()).getExtendedKeyUsage());
        }
        catch (IOException cause) {
            throw new CertificateParsingException(cause);
        }
    }
    
    @Override
    public int getBasicConstraints() {
        try {
            String name = OIDMap.getName(PKIXExtensions.BasicConstraints_Id);
            if (name == null) {
                return -1;
            }
            BasicConstraintsExtension basicConstraintsExtension = (BasicConstraintsExtension)this.get(name);
            if (basicConstraintsExtension == null) {
                return -1;
            }
            if ((boolean)basicConstraintsExtension.get("is_ca")) {
                return (int)basicConstraintsExtension.get("path_len");
            }
            return -1;
        }
        catch (Exception ex) {
            return -1;
        }
    }
    
    private static Collection<List<?>> makeAltNames(GeneralNames generalNames) {
        if (generalNames.isEmpty()) {
            return Collections.emptySet();
        }
        ArrayList<List<Object>> c = new ArrayList();
        Iterator<GeneralName> iterator = generalNames.names().iterator();
        while (iterator.hasNext()) {
            GeneralNameInterface name = ((GeneralName)iterator.next()).getName();
            ArrayList list = new ArrayList(2);
            list.add(Integer.valueOf(name.getType()));
            switch (name.getType()) {
                case 1: {
                    list.add(((RFC822Name)name).getName());
                    break;
                }
                case 2: {
                    list.add(((DNSName)name).getName());
                    break;
                }
                case 4: {
                    list.add(((X500Name)name).getRFC2253Name());
                    break;
                }
                case 6: {
                    list.add(((URIName)name).getName());
                    break;
                }
                case 7: {
                    try {
                        list.add(((IPAddressName)name).getName());
                        break;
                    }
                    catch (IOException cause) {
                        throw new RuntimeException("IPAddress cannot be parsed", cause);
                    }
                }
                case 8: {
                    list.add(((OIDName)name).getOID().toString());
                    break;
                }
                default: {
                    DerOutputStream o = new DerOutputStream();
                    try {
                        name.encode(o);
                    }
                    catch (IOException cause2) {
                        throw new RuntimeException("name cannot be encoded", cause2);
                    }
                    list.add(o.toByteArray());
                    break;
                }
            }
            c.add(Collections.unmodifiableList(list));
        }
        return Collections.unmodifiableCollection(c);
    }
    
    private static Collection<List<?>> cloneAltNames(Collection<List<?>> collection) {
        boolean n = false;
        Iterator<Object> iterator = collection.iterator();
        while (iterator.hasNext()) {
            if ((((List)iterator.next()).get(1) instanceof byte[])) {
                n = true;
            }
            else {
                continue;
            }
        }
        if (!(n == 0)) {
            ArrayList c = new ArrayList();
            for (List c2 : collection) {
                Object value = c2.get(1);
                if ((value instanceof byte[])) {
                    ArrayList list = new ArrayList(c2);
                    list.set(1, ((byte[])(byte[])value).clone());
                    c.add(Collections.unmodifiableList(list));
                }
                else {
                    c.add(c2);
                }
            }
            return Collections.unmodifiableCollection(c);
        }
        return collection;
    }
    
    @Override
    public synchronized Collection<List<?>> getSubjectAlternativeNames() throws CertificateParsingException {
        if (this.readOnly && this.subjectAlternativeNames != null) {
            return cloneAltNames(this.subjectAlternativeNames);
        }
        SubjectAlternativeNameExtension subjectAlternativeNameExtension = this.getSubjectAlternativeNameExtension();
        if (subjectAlternativeNameExtension == null) {
            return null;
        }
        GeneralNames value;
        try {
            value = subjectAlternativeNameExtension.get("subject_name");
        }
        catch (IOException ex) {
            return Collections.emptySet();
        }
        return this.subjectAlternativeNames = makeAltNames(value);
    }
    
    public static Collection<List<?>> getSubjectAlternativeNames(X509Certificate x509Certificate) throws CertificateParsingException {
        try {
            byte[] extensionValue = x509Certificate.getExtensionValue("2.5.29.17");
            if (extensionValue == null) {
                return null;
            }
            SubjectAlternativeNameExtension subjectAlternativeNameExtension = new SubjectAlternativeNameExtension(Boolean.FALSE, new DerValue(extensionValue).getOctetString());
            GeneralNames value;
            try {
                value = subjectAlternativeNameExtension.get("subject_name");
            }
            catch (IOException ex) {
                return Collections.emptySet();
            }
            return makeAltNames(value);
        }
        catch (IOException cause) {
            throw new CertificateParsingException(cause);
        }
    }
    
    @Override
    public synchronized Collection<List<?>> getIssuerAlternativeNames() throws CertificateParsingException {
        if (this.readOnly && this.issuerAlternativeNames != null) {
            return cloneAltNames(this.issuerAlternativeNames);
        }
        IssuerAlternativeNameExtension issuerAlternativeNameExtension = this.getIssuerAlternativeNameExtension();
        if (issuerAlternativeNameExtension == null) {
            return null;
        }
        GeneralNames value;
        try {
            value = issuerAlternativeNameExtension.get("issuer_name");
        }
        catch (IOException ex) {
            return Collections.emptySet();
        }
        return this.issuerAlternativeNames = makeAltNames(value);
    }
    
    public static Collection<List<?>> getIssuerAlternativeNames(X509Certificate x509Certificate) throws CertificateParsingException {
        try {
            byte[] extensionValue = x509Certificate.getExtensionValue("2.5.29.18");
            if (extensionValue == null) {
                return null;
            }
            IssuerAlternativeNameExtension issuerAlternativeNameExtension = new IssuerAlternativeNameExtension(Boolean.FALSE, new DerValue(extensionValue).getOctetString());
            GeneralNames value;
            try {
                value = issuerAlternativeNameExtension.get("issuer_name");
            }
            catch (IOException ex) {
                return Collections.emptySet();
            }
            return makeAltNames(value);
        }
        catch (IOException cause) {
            throw new CertificateParsingException(cause);
        }
    }
    
    public AuthorityInfoAccessExtension getAuthorityInfoAccessExtension() {
        return (AuthorityInfoAccessExtension)this.getExtension(PKIXExtensions.AuthInfoAccess_Id);
    }
    
    private void parse(DerValue derValue) throws CertificateException, IOException {
        if (this.readOnly) {
            throw new CertificateParsingException("cannot over-write existing certificate");
        }
        if (derValue.data == null || derValue.tag != 48) {
            throw new CertificateParsingException("invalid DER-encoded certificate data");
        }
        this.signedCert = derValue.toByteArray();
        DerValue[] array = { derValue.data.getDerValue(), derValue.data.getDerValue(), derValue.data.getDerValue() };
        if (derValue.data.available() != 0) {
            throw new CertificateParsingException("signed overrun, bytes = " + derValue.data.available());
        }
        if (array[0].tag != 48) {
            throw new CertificateParsingException("signed fields invalid");
        }
        this.algId = AlgorithmId.parse(array[1]);
        this.signature = array[2].getBitString();
        if (array[1].data.available() != 0) {
            throw new CertificateParsingException("algid field overrun");
        }
        if (array[2].data.available() != 0) {
            throw new CertificateParsingException("signed fields overrun");
        }
        this.info = new X509CertInfo(array[0]);
        if (!this.algId.equals((AlgorithmId)this.info.get("algorithmID.algorithm"))) {
            throw new CertificateException("Signature algorithm mismatch");
        }
        this.readOnly = true;
        return;
    }
    
    private static X500Principal getX500Principal(X509Certificate x509Certificate, boolean b) throws Exception {
        DerInputStream data = new DerInputStream(x509Certificate.getEncoded()).getSequence(3)[0].data;
        if (data.getDerValue().isContextSpecific((byte)0)) {
            data.getDerValue();
        }
        data.getDerValue();
        DerValue derValue = data.getDerValue();
        if (!b) {
            data.getDerValue();
            derValue = data.getDerValue();
        }
        return new X500Principal(derValue.toByteArray());
    }
    
    public static X500Principal getSubjectX500Principal(X509Certificate x509Certificate) {
        try {
            return getX500Principal(x509Certificate, false);
        }
        catch (Exception cause) {
            throw new RuntimeException("Could not parse subject", cause);
        }
    }
    
    public static X500Principal getIssuerX500Principal(X509Certificate x509Certificate) {
        try {
            return getX500Principal(x509Certificate, true);
        }
        catch (Exception cause) {
            throw new RuntimeException("Could not parse issuer", cause);
        }
    }
    
    public static byte[] getEncodedInternal(Certificate certificate) throws CertificateEncodingException {
        if ((certificate instanceof X509CertImpl)) {
            return ((X509CertImpl)certificate).getEncodedInternal();
        }
        return certificate.getEncoded();
    }
    
    public static X509CertImpl toImpl(X509Certificate c) throws CertificateException {
        if ((c instanceof X509CertImpl)) {
            return (X509CertImpl)c;
        }
        return X509Factory.intern(c);
    }
    
    public static boolean isSelfIssued(X509Certificate x509Certificate) {
        return x509Certificate.getSubjectX500Principal().equals(x509Certificate.getIssuerX500Principal());
    }
    
    public static boolean isSelfSigned(X509Certificate x509Certificate, String s) {
        if (isSelfIssued(x509Certificate)) {
            try {
                if (s == null) {
                    x509Certificate.verify(x509Certificate.getPublicKey());
                }
                else {
                    x509Certificate.verify(x509Certificate.getPublicKey(), s);
                }
                return true;
            }
            catch (Exception ex) {}
        }
        return false;
    }
    
    public String getFingerprint(String key) {
        return (String)this.fingerprints.computeIfAbsent(key, s/* java.lang.String, */ -> getFingerprint(s, this));
    }
    
    public static String getFingerprint(String algorithm, X509Certificate x509Certificate) {
        String string = "";
        try {
            byte[] digest = MessageDigest.getInstance(algorithm).digest(x509Certificate.getEncoded());
            StringBuffer sb = new StringBuffer();
            for (int i = 0; i < digest.length; ++i) {
                byte2hex(digest[i], sb);
            }
            string = sb.toString();
        }
        catch (NoSuchAlgorithmException | CertificateEncodingException ex) {}
        return string;
    }
    
    private static void byte2hex(byte b, StringBuffer sb) {
        char[] array = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
        int n = (b & 0xF0) >> 4;
        int n2 = b & 0xF;
        sb.append(array[n]);
        sb.append(array[n2]);
    }
}
